from __future__ import unicode_literals

import chargehound
import requests
from chargehound.error import create_chargehound_error
from chargehound.version import VERSION


class APIRequestor(object):

    def parse_response(self, response):
        payload = response.json()
        if response.status_code == 200:
            return payload
        else:
            raise create_chargehound_error(payload)

    def handle_callback(self, callback):
        def handle_response(response,  **kwargs):
            parsed = self.parse_response(response)
            callback(parsed)
        return handle_response

    def get_url(self, path):
        return 'https://' + chargehound.host + chargehound.base_path + path

    def make_request(self, method, path, params=None, data=None,
                     callback=None):
        headers = {
            'accept': 'application/json',
            'user-agent': 'Chargehound/v1 PythonBindings/%s' % VERSION
        }

        auth = (chargehound.api_key, '')

        if callback:
            hooks = dict(response=self.handle_callback(callback))
        else:
            hooks = None

        if method == 'get':
            return self.parse_response(requests.get(self.get_url(path),
                                       auth=auth,
                                       params=params,
                                       headers=headers,
                                       hooks=hooks))
        elif method == 'post':
            return self.parse_response(requests.post(self.get_url(path),
                                       auth=auth,
                                       json=data,
                                       headers=headers,
                                       hooks=hooks))

    def request(self, method, path, params=None, data=None, callback=None):
        if callback is None:
            return self.make_request(method, path, params, data)
        else:
            return self.make_request(method, path, params, data, callback)



from chargehound.api_requestor import APIRequestor


class Disputes(object):

    """
    Retrieve a dispute
    This method will return a single dispute.

    :param str dispute_id: A dispute id (required)
    :param callback function: The callback function
        to handle the response. (optional)
    :return: Dispute
    """
    @classmethod
    def retrieve(klass, dispute_id, callback=None):
        requestor = APIRequestor()
        return requestor.request('get', 'disputes/{0}'.format(dispute_id),
                                 callback=callback)

    """
    A list of disputes
    This method will list all the disputes that we have synced from Stripe.
    By default the disputes will be ordered by `created`
    with the most recent dispute first.
    `has_more` will be `true` if more results are available.

    :param int limit: Maximum number of disputes to return.
        Default is 20, maximum is 100. (optional)
    :param str starting_after: A dispute id.
        Fetch disputes created after this dispute. (optional)
    :param str ending_before: A dispute id.
        Fetch disputes created before this dispute. (optional)
    :param callback function: The callback function
        to handle the response. (optional)
    :return: Disputes
    """
    @classmethod
    def list(klass, **kwargs):
        callback = kwargs.pop('callback', None)
        list_params = kwargs

        requestor = APIRequestor()
        return requestor.request('get', 'disputes',
                                 params=list_params,
                                 callback=callback)

    """
    Submitting a dispute
    You will want to submit the dispute through Chargehound after you recieve
    a notification from Stripe of a new dispute.
    With one `POST` request you can update a dispute with the
    evidence fields and send the generated response to Stripe.
    The response will have a `201` status if the submit was successful.
    The dispute will also be in the submitted state.

    :param str dispute_id: A dispute id (required)
    :param str template: Set the template for this dispute. (optional)
    :param object fields:
        Key value pairs to hydrate the template's evidence fields. (optional)
    :param str customer_name: Update the customer name.
        Will also update the customer name in the evidence fields. (optional)
    :param str customer_email:
        Update the customer email. Will also update the customer email
        in the evidence fields. Must be a valid email address. (optional)
    :param callback function: The callback function
        to handle the response. (optional)
    :return: Dispute
    """
    @classmethod
    def submit(klass, dispute_id, **kwargs):
        callback = kwargs.pop('callback', None)
        update = kwargs

        requestor = APIRequestor()
        return requestor.request('post',
                                 'disputes/{0}/submit'.format(dispute_id),
                                 data=update,
                                 callback=callback)

    """
    Updating a dispute
    You can update the template and the fields on a dispute.

    :param str dispute_id: A dispute id (required)
    :param str template: Set the template for this dispute. (optional)
    :param object fields:
        Key value pairs to hydrate the template's evidence fields. (optional)
    :param str customer_name: Update the customer name.
        Will also update the customer name in the evidence fields. (optional)
    :param str customer_email:
        Update the customer email. Will also update the customer email
        in the evidence fields. Must be a valid email address. (optional)
    :param callback function: The callback function
        to handle the response. (optional)
    :return: Dispute
    """
    @classmethod
    def update(klass, dispute_id, **kwargs):
        callback = kwargs.pop('callback', None)
        update = kwargs

        requestor = APIRequestor()
        return requestor.request('post', 'disputes/{0}'.format(dispute_id),
                                 data=update,
                                 callback=callback)



import chargehound
import requests_mock
import unittest2


class ChargehoundTest(unittest2.TestCase):

    @requests_mock.mock()
    def test_override_host(self, mock):
        orig_host = chargehound.host
        chargehound.host = 'test'
        mock.get('https://test/v1/disputes/dp_123',
                 json={'id': 'dp_123'})
        chargehound.Disputes.retrieve('dp_123')
        assert mock.called
        chargehound.host = orig_host



import chargehound
import requests_mock
import unittest2
from chargehound.error import ChargehoundBadRequestError, ChargehoundError


class ErrorTest(unittest2.TestCase):

    def setUp(self):
        super(ErrorTest, self).setUp()
        chargehound.api_key = 'API_KEY'

    @requests_mock.mock()
    def test_bad_request(self, mock):
        mock.post('https://api.chargehound.com/v1/disputes/dp_123/submit',
                  status_code=400,
                  json={
                    'error': {'status': 400, 'message': 'Bad request.'}
                  })

        try:
            chargehound.Disputes.submit('dp_123')
        except ChargehoundBadRequestError as bad:
            assert bad.status == 400
            assert bad.message == 'Bad request.'

    def test_propagate_errors(self):
        orig_host = chargehound.host
        chargehound.host = 'test'

        try:
            chargehound.Disputes.retrieve('dp_123')
        except Exception as e:
            assert not isinstance(e, ChargehoundError)
            chargehound.host = orig_host



import chargehound
import requests_mock
import unittest2
from chargehound.version import VERSION


get_headers = {
    'accept': 'application/json',
    'authorization': 'Basic QVBJX0tFWTo=',
    'user-agent': 'Chargehound/v1 PythonBindings/' + VERSION
}

post_headers = {
    'accept': 'application/json',
    'authorization': 'Basic QVBJX0tFWTo=',
    'content-type': 'application/json',
    'user-agent': 'Chargehound/v1 PythonBindings/' + VERSION
}


class DisputeTest(unittest2.TestCase):

    def setUp(self):
        super(DisputeTest, self).setUp()
        chargehound.api_key = 'API_KEY'

    @requests_mock.mock()
    def test_retrieve_dispute(self, mock):
        mock.get('https://api.chargehound.com/v1/disputes/dp_123',
                 request_headers=get_headers,
                 json={'id': 'dp_123'})
        chargehound.Disputes.retrieve('dp_123')
        assert mock.called

    @requests_mock.mock()
    def test_list_all_disputes(self, mock):
        mock.get('https://api.chargehound.com/v1/disputes',
                 request_headers=get_headers,
                 json={'data': [{'id': 'dp_123'}]})
        chargehound.Disputes.list()
        assert mock.called

    @requests_mock.mock()
    def test_submit_dispute(self, mock):
        mock.post('https://api.chargehound.com/v1/disputes/dp_123/submit',
                  json={'id': 'dp_123'})
        chargehound.Disputes.submit('dp_123',
                                    fields={'customer_name': 'Susie'})
        assert mock.request_history[0].body == \
            '{"fields": {"customer_name": "Susie"}}'
        assert mock.called

    @requests_mock.mock()
    def test_update_dispute(self, mock):
        mock.post('https://api.chargehound.com/v1/disputes/dp_123',
                  json={'id': 'dp_123'})
        chargehound.Disputes.update('dp_123',
                                    fields={'customer_name': 'Susie'})
        assert mock.request_history[0].body == \
            '{"fields": {"customer_name": "Susie"}}'
        assert mock.called



try:
    from setuptools import setup
except ImportError:
    from disutils.core import setup

setup(
    name='chargehound',
    version='1.0.1',
    author='Chargehound',
    author_email='support@chargehound.com',
    packages=['chargehound'],
    description='Chargehound Python Bindings',
    url='https://www.chargehound.com',
    license='MIT',
    test_suite='test.all',
    install_requires=[
        'requests'
        ],
    )
